/*
******************************************************************************

   LibOVL: extension internals

   Copyright (C) 2001 Christoph Egger   [Christoph_Egger@t-online.de]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _GGI_OVL_INT_H
#define _GGI_OVL_INT_H

#include <ggi/internal/internal.h>
#include <ggi/galloc.h>


#define HAVE_STRUCT_OVL
#include <ggi/ovl.h>

__BEGIN_DECLS

/* Exported variables */
#ifdef BUILDING_LIBOVL
extern ggi_extid    _ovlID;
#else
IMPORTVAR ggi_extid _ovlID;
#endif

__END_DECLS



/* Internal data types */

struct Common {
	/* pointer to a directbuffer.
	 * Note, that this may be shared with other sprites.
	 * If direct access to the sprite has to be emulated,
	 * these functions should not use the read and
	 * write members of this db-struct.
	 */
	const ggi_directbuffer *db;

	/* Pointers to the VRAM adress, if the sprite data
	 * can be accessed directly. Otherwise, they point
	 * to some RAM allocated from system memory.
	 */
	void *read;
	void *write;

	/* Pointer to set/get functions.  These functions will
	 * transfer the contents of a system RAM buffer to/from the
	 * sprite.  If you are writing to VRAM directly, the function
	 * will do nothing, but returning OVL_EDOESNOTHING.
	 */
	int (*setmem)(ggiOvl_t ovl, void *buf);
	int (*getmem)(ggiOvl_t ovl, void *buf);


	/* Pointer to set/get functions. These functions will
	 * transfer the contents of a VRAM buffer to/from the sprite
	 * If you are writing to a system RAM buffer, the function
	 * will do nothing, but returning OVL_EDOESNOTHING.
	 */
	int (*setdb)(ggiOvl_t ovl, void *buf);
	int (*getdb)(ggiOvl_t ovl, void *buf);
};


struct Sprite {
	/* You can add target independent datatypes for the
	 * properties here
	 */

	/* target specific stuff */
	void *target;
};

struct Video {
	/* You can add target independent datatypes for the
	 * properties here
	 */

	/* target specific stuff */
	void *target;
};

struct Window {
	/* You can add target independent datatypes for the
	 * properties here
	 */

	/* target specific stuff */
	void *target;
};


struct ggiOvl {

	struct ggiGA_resource_props *props;

	ggiGA_resource_handle res_handle_tank,  req_handle_tank;
	ggiGA_resource_handle res_handle_carb,  req_handle_carb;
	ggiGA_resource_handle res_handle_motor, req_handle_motor;

	int motor_count;	/* usage counter of the motor */
	int carb_count;		/* usage counter of the carb */

	enum ggiGA_resource_type res_type;
	ggi_visual_t vis;
	long x,y;	/* current position */
	long w,h;	/* the size of the STORAGE */

	/* current pos/size - needed for pixel-doubling and zooming */
	long cur_x, cur_y;
	long cur_w, cur_h;


	int hidecount;
	int disablecount;


	uint32 slot;
	uint32 flags;
	int zoomed;


	void *alpha;
	size_t alpha_size;

	/* The coordbase the resource is actually stored
	 * and handled by the target
	 */
	uint16 cb;

	/* This is the coordbase the user is working with.
	 * Can be any kind of supported coordbase.
	 */
	uint16 user_cb;

	struct Common common;
	union {
		struct Sprite sprite;
		struct Video  video;
		struct Window window;
	} Ovltype;
};


#define ggiOvlDirectAccess(ovl)		(ovl->common.db != NULL)


#define ggiOvlSetMemData(ovl, buf)	ovl->common.setmem(ovl, buf);
#define ggiOvlGetMemData(ovl, buf)	ovl->common.getmem(ovl, buf);

#define ggiOvlSetDBData(ovl, buf)	ovl->common.setdb(ovl, buf);
#define ggiOvlGetDBData(ovl, buf)	ovl->common.getdb(ovl, buf);




/* Data types for the API functions */
typedef int (OvlTie)(ggi_visual_t vis, ggiOvl_t ovl);

typedef int (OvlSetImage)(ggiOvl_t ovl, ggi_visual_t vis,
			long x, long y, unsigned char *alpha);


typedef int (OvlSetPos)(ggiOvl_t ovl, long x, long y);
typedef int (OvlMove)(ggiOvl_t ovl, long x, long y);

typedef int (OvlSetSize)(ggiOvl_t ovl, long w, long h);
typedef int (OvlSetPriority)(ggiOvl_t ovl, uint32 slot, int swap);
typedef int (OvlSetFlags)(ggiOvl_t ovl, enum ggiOvl_flags flags);

typedef int (OvlSetZoom)(ggiOvl_t ovl, long x, long y,
			long w, long h, int keep_ratio);


typedef int (OvlShow)(ggiOvl_t ovl);
typedef int (OvlHide)(ggiOvl_t ovl);

typedef int (OvlEnable)(ggiOvl_t ovl);
typedef int (OvlDisable)(ggiOvl_t ovl);


typedef struct ovlpriv {
	OvlTie		*tie;

	OvlSetImage	*setimage;

	OvlSetPos	*setpos;
	OvlMove		*move;

	OvlSetSize	*setsize;
	OvlSetPriority	*setpriority;
	OvlSetFlags	*setflags;

	OvlSetZoom	*setzoom;

	OvlShow		*show;
	OvlHide		*hide;

	OvlEnable	*enable;
	OvlDisable	*disable;




	
	ggiGA_resource_list *reqlist;  /* Pointer to application provided 
					  "global" request list for lazy API */

	void *priv;	/* Here sublibs can store visual specific private data.
			   If you have more than one sublib (for each visual)
			   that needs private data, just add more pointers or
			   add the data fields directly in this structure. */
} ovlpriv;

/* Used to access the ovlpriv structure for a particular visual. */
#define LIBGGI_OVLEXT(vis) ((ovlpriv *)LIBGGI_EXT((vis),_ovlID))



int _ggiOvlSync(ggiOvl_t ovl);


#endif /* _GGI_OVL_INT_H */
